@@ -7,6 +7,9 @@ import logging
 import shutil
 import shlex
 from string import Template
+import string
+import random
+
 
 LOGGER = logging.getLogger('playbook_runner')
 
@@ -28,16 +31,6 @@ class AnsiblePlaybook(object):
         self._hosts = set()
         self._cleanup_output = cleanup_output
 
-        hosts = self._get_hosts_by_group(
-            self._ansible_playbook_inventory,
-            'all'
-        )
-
-        hosts.append('localhost')
-        hosts.append('127.0.0.1')
-
-        self._hosts.update(hosts)
-
         LOGGER.info('Output path: {0}'.format(self._path_str))
         LOGGER.info('CWD: {0}'.format(self._ansible_playbook_directory))
 
@@ -53,8 +46,7 @@ class AnsiblePlaybook(object):
                 )
 
     @staticmethod
-    def _get_ansible_cmd(inventory_file, playbook_file,
-                        extra_vars_dict=None):
+    def _get_ansible_cmd(inventory_file, playbook_file, extra_vars_dict):
         """
         Return process args list for ansible-playbook run.
         """
@@ -62,22 +54,22 @@ class AnsiblePlaybook(object):
             "ansible-playbook",
             "-vv",
             "--fork",
-            "50",
+            extra_vars_dict['fork_factor'],
             "--timeout", "60",
             "-i", inventory_file,
             playbook_file,
         ]
-        if extra_vars_dict:
-            extra_vars = ''
-            for k, v in extra_vars_dict.items():
-                if type(v) == str:
-                    extra_vars += '{}="{}" '.format(k, v)
-                else:
-                    extra_vars += '{}="{}" '.format(k, v)
 
-            extra_vars = extra_vars[:-1]
-            ansible_command.insert(-1, '--extra-vars')
-            ansible_command.insert(-1, extra_vars)
+        extra_vars = ''
+        for k, v in extra_vars_dict.items():
+            if type(v) == str:
+                extra_vars += '{}="{}" '.format(k, v)
+            else:
+                extra_vars += '{}="{}" '.format(k, v)
+
+        extra_vars = extra_vars[:-1]
+        ansible_command.insert(-1, '--extra-vars')
+        ansible_command.insert(-1, extra_vars)
 
         return ansible_command
 
@@ -142,8 +134,12 @@ class AnsiblePlaybook(object):
 
         return parent_path
 
-
     def run_playbook(self, play_filename, extra_vars_dict=None):
+        random_run_id = ''.join(
+            random.choices(
+                string.ascii_uppercase + string.digits, k=64
+            )
+        )
         if not extra_vars_dict:
             extra_vars_dict = {}
 
@@ -155,6 +151,24 @@ class AnsiblePlaybook(object):
             local_extra_vars['gather_facts_for_pb'] = False
         if 'play_host_groups' not in local_extra_vars:
             local_extra_vars['play_host_groups'] = 'localhost'
+        if 'fork_factor' not in local_extra_vars:
+            local_extra_vars['fork_factor'] = 50
+
+        if not self._hosts:
+            local_hosts = ['localhost', '127.0.0.1']
+            hosts = self._get_hosts_by_group(
+                self._ansible_playbook_inventory,
+                'all'
+            )
+
+            group = local_extra_vars['play_host_groups']
+            if not hosts and group not in local_hosts:
+                raise RuntimeError(
+                    'Failed to list hosts. Will use localhost only'
+                )
+
+            hosts.extend(local_hosts)
+            self._hosts.update(hosts)
 
         path = self._generate_parent_play(self._ansible_playbook_directory, play_filename)
         cmd = self._get_ansible_cmd(
@@ -170,9 +184,6 @@ class AnsiblePlaybook(object):
                 with open(file_path, 'w') as f:
                     f.write('[')
 
-        LOGGER.info(
-            'Command is about to be run:\n{0}'.format(' '.join(cmd_for_log))
-        )
         ansible_output_path = '{0}/ansible_output_path.txt'.format(
             self._path_str
         )
@@ -181,9 +192,11 @@ class AnsiblePlaybook(object):
         our_env["ANSIBLE_HOST_KEY_CHECKING"] = "False"
         with open(ansible_output_path, "a+") as f_ansible_output_path:
             f_ansible_output_path.write(
-                '\n\nGoing to run ansible playbook:\n{0}\n\n'.format(
-                    ' '.join(cmd_for_log)
-                    )
+                '\n\nGoing to run ansible playbook:\n'
+                '{0}\nrun_id:{1}\n\n'.format(
+                    ' '.join(cmd_for_log),
+                    random_run_id
+                )
             )
 
             result = subprocess.run(
@@ -198,7 +211,10 @@ class AnsiblePlaybook(object):
             if not local_extra_vars['skip_errors']:
                 if result.returncode != 0:
                     LOGGER.error(
-                        'Failed to run:\n{0}'.format(' '.join(cmd_for_log))
+                        'Failed to run:\n{0}\nrun_id: {1}'.format(
+                            ' '.join(cmd_for_log),
+                            random_run_id
+                        )
                     )
 
         return result.returncode
